#include <linux/syno.h>

#include <linux/module.h>
#include <linux/kernel.h> /* printk() */
#include <linux/errno.h>  /* error codes */
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/genhd.h>
#include <linux/major.h>
#include <asm/uaccess.h>
#include <linux/poll.h>
#include <linux/delay.h>
#include <linux/pm.h>
//#include <linux/smp_lock.h>
#include <linux/sched.h>

#include <linux/synobios.h>
#include <linux/ioport.h>
#include "mapping.h"

#ifdef MY_ABC_HERE
#include <linux/raid/libmd-report.h>
#endif

#if 1
#define	DBGMESG(x...)	printk(x)
#else
#define	DBGMESG(x...)
#endif
/*
// Synology DS213j / WDMC Mirror
static module_t dummymodule = {
	FAN_ADT, // or FAN_MULTI_EXCEPT_SLEEP
	RAID_ON,
	LED_DISKS_ALARM,
	THERMAL_YES,
	DISK_LED_CTRL_HW,    
	AUTO_POWERON_NO,
	POWER_SINGLE,
	USBCOPY_NO,
	FAN_NUMBER_1,
	EUNIT_PWRON_GPIO,		// ?
	POWER_IN_SEQ_OFF,
	RTC_MV,					// RTC Marvell
	CPUTMP_YES,
	FAN_RPM_RPT_YES,		// Fan RPM reporting
	LCM_UNKNOWN,
	WIFI_WPS_NO,
	CPU_FREQ_ADJUST_YES,
	CARDREADER_NO,
	HIBER_LED_STATUS_BREATH,
	RTC_NOT_NEED_TO_CORRECT,
	MICROP_ID_UNKNOW,		// No MicroP
	ONE_DISK_DENO_ONE,
	CPU_MV78230,	// Armada XP series
	CRYPTO_A370,	// Armada 370
};
*/

static module_t dummymodule = {
	FAN_UNKNOWN,
	RAID_UNKNOWN,
	LED_UNKNOWN,
	THERMAL_UNKNOWN,
	DISK_LED_CTRL_UNKNOWN,
	AUTO_POWERON_UNKNOWN,
	POWER_UNKNOWN,
	USBCOPY_NO,
	FAN_NUMBER_X,
	EUNIT_NOT_SUPPORT,
	POWER_IN_SEQ_OFF,
	RTC_UNKNOWN,
	CPUTMP_UNKNOWN,
	FAN_RPM_RPT_UNKNOWN,
	LCM_UNKNOWN,
	WIFI_WPS_UNKNOWN,
	CPU_FREQ_ADJUST_UNKNOWN,
	CARDREADER_UNKNOWN,
	HIBER_LED_UNKNOWN,
	RTC_NOT_NEED_TO_CORRECT,
	MICROP_ID_UNKNOW,
	UNKNOW_DISK_DENO,
	CPU_UNKNOWN,
	CRYPTO_HW_NONE,
};

static unsigned int synobios_poll(struct file *pfile, struct poll_table_struct *ppolltable)
{
	int revents = 0;

	return (revents);
}							

static long synobios_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	long ret = 0;

	if (_IOC_TYPE(cmd) != SYNOBIOS_IOC_MAGIC) {
		printk(KERN_INFO "synobios_ioctl: type mismatch %x\n", cmd);
		ret = -ENOTTY;
		goto END;
	}

	switch (cmd) {
	case SYNOIO_NEXTEVENT:
		DBGMESG("synobios_ioctl: SYNOIO_NEXTEVENT\n");
		ret = -EAGAIN;
		break;
	case SYNOIO_RTC_CONTROL_READ:
		DBGMESG("synobios_ioctl: SYNOIO_RTC_CONTROL_READ\n");
		ret = -1;
		break;
	case SYNOIO_RTC_CONTROL_WRITE:
		DBGMESG("synobios_ioctl: SYNOIO_RTC_CONTROL_WRITE\n");
		ret = -1;
		break;
	case SYNOIO_RTC_TIME_READ:
		DBGMESG("synobios_ioctl: SYNOIO_RTC_TIME_READ\n");
		ret = -1;
		break;
	case SYNOIO_RTC_TIME_WRITE:
		DBGMESG("synobios_ioctl: SYNOIO_RTC_TIME_WRITE\n");
		ret = -1;
		break;
	case SYNOIO_MANUTIL_MODE:
		DBGMESG("synobios_ioctl: SYNOIO_MANUTIL_MODE\n");
		ret = 0;
		break;
	case SYNOIO_RECORD_EVENT:
		printk(KERN_INFO "synobios_ioctl: SYNOIO_RECORD_EVENT, event id %x\n", *((u_int *)arg));
		break;
	case SYNOIO_BUTTON_RESET:
		DBGMESG("synobios_ioctl: SYNOIO_BUTTON_RESET\n");
		printk("synobios: reset button pressed, ret = %ld\n", ret);
		break;
	case SYNOIO_BUTTON_POWER:
		DBGMESG("synobios_ioctl: SYNOIO_BUTTON_POWER\n");
		printk("synobios: power button pressed, ret = %ld\n", ret);
		break;
	case SYNOIO_BUTTON_USB:
		DBGMESG("synobios_ioctl: SYNOIO_BUTTON_USB\n");
		printk("synobios: usb button pressed, ret = %ld\n", ret);
		break;
	case SYNOIO_SET_DISK_LED: // Called repeatedly
		//DBGMESG("synobios_ioctl: SYNOIO_SET_DISK_LED\n");
		ret = 0;
		break;
	case SYNOIO_GET_FAN_STATUS:
		DBGMESG("synobios_ioctl: SYNOIO_GET_FAN_STATUS\n");
		{
			FANSTATUS*  pFanStatus = (FANSTATUS *)arg;
			pFanStatus->status = FAN_STATUS_RUNNING;
			ret = 0;
		}
		break;
	case	SYNOIO_SET_FAN_STATUS:
		DBGMESG("synobios_ioctl: SYNOIO_SET_FAN_STATUS\n");
		ret = 0;
		break;
	case	SYNOIO_GET_FAN_NUM:
		DBGMESG("synobios_ioctl: SYNOIO_GET_FAN_NUM\n");
		ret = 0;
		break;
	case	SYNOIO_GET_DS_BRAND:
		DBGMESG("synobios_ioctl: SYNOIO_GET_DS_BRAND\n");
		{
			int *pBrand = (int *)arg;
			*pBrand = BRAND_SYNOLOGY;
			break;
		}
	case	SYNOIO_GET_DS_MODEL:
		DBGMESG("synobios_ioctl: SYNOIO_GET_DS_MODEL\n");
		{
			int* pModel = (int *)arg;
			*pModel = MODEL_DS215j;
			break;
		}
	case	SYNOIO_GET_CPLD_VERSION:
		DBGMESG("synobios_ioctl: SYNOIO_GET_CPLD_VERSION\n");
		{
			int *pVersion = (int *)arg;
			*pVersion = 0;
			ret = 0;
			break;
		}
	case	SYNOIO_GET_TEMPERATURE:
		DBGMESG("synobios_ioctl: SYNOIO_GET_TEMPERATURE\n");
		{
			int *Temperature = (int *)arg;
			*Temperature = 35;
			ret = 0;
			break;
		}
	case    SYNOIO_GET_CPLD_REG:
		DBGMESG("synobios_ioctl: SYNOIO_GET_CPLD_REG\n");
		{
			ret = -1;
			break;
		}
	case    SYNOIO_SET_MEM_BYTE:
		DBGMESG("synobios_ioctl: SYNOIO_SET_MEM_BYTE\n");
		{
			ret = -1;
			break;
		}
	case    SYNOIO_GET_MEM_BYTE:
		DBGMESG("synobios_ioctl: SYNOIO_GET_MEM_BYTE\n");
		{
			ret = -1;
			break;
		}
	case    SYNOIO_GPIO_PIN_WRITE:
		DBGMESG("synobios_ioctl: SYNOIO_GPIO_PIN_WRITE\n");
		{
			ret = -1;
			break;
		}
	case    SYNOIO_GPIO_PIN_READ:
		DBGMESG("synobios_ioctl: SYNOIO_GPIO_PIN_READ\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_GET_AUTO_POWERON:
		DBGMESG("synobios_ioctl: SYNOIO_GET_AUTO_POWERON\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_SET_AUTO_POWERON:
		DBGMESG("synobios_ioctl: SYNOIO_SET_AUTO_POWERON\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_GET_HW_CAPABILITY:
		DBGMESG("synobios_ioctl: SYNOIO_GET_HW_CAPABILITY\n");
		{
			//CAPABILITY *pCapability = (CAPABILITY *)arg;
			//ret = GetHwCapability(pCapability);
			ret = 0;
			break;
		}
	case SYNOIO_SET_ALARM_LED:
		DBGMESG("synobios_ioctl: SYNOIO_SET_ALARM_LED\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_GET_BUZZER_CLEARED:
		DBGMESG("synobios_ioctl: SYNOIO_GET_BUZZER_CLEARED\n");
		{
			ret = 0;
			break;
		}
	case SYNOIO_GET_POWER_STATUS:
		DBGMESG("synobios_ioctl: SYNOIO_GET_POWER_STATUS\n");
		{
			ret = 0;
			break;
		}
	case SYNOIO_SHUTDOWN_LOG:
		DBGMESG("synobios_ioctl: SYNOIO_SHUTDOWN_LOG\n");
		{
			//int event = (SYNO_SHUTDOWN_LOG)arg;
			break;
		}
	case SYNOIO_UNINITIALIZE:
		DBGMESG("synobios_ioctl: SYNOIO_UNINITIALIZE\n");
		{
			ret = 0;
			break;
		}
	case SYNOIO_GET_SYS_STATUS:
		DBGMESG("synobios_ioctl: SYNOIO_GET_SYS_STATUS\n");
		{
			//SYNO_SYS_STATUS *pUSysStat = (SYNO_SYS_STATUS *)arg;
			ret = -1;
			break;
		}
	case SYNOIO_SET_SYS_STATUS:
		DBGMESG("synobios_ioctl: SYNOIO_SET_SYS_STATUS\n");
		{
			sys_comp_stat_t uSysStat = (sys_comp_stat_t)arg;
			break;
		}
	case SYNOIO_GET_MODULE_TYPE:
		DBGMESG("synobios_ioctl: SYNOIO_GET_MODULE_TYPE\n");
		{
			copy_to_user((void __user *)arg, &dummymodule, sizeof(module_t));
			break;
		}
	case SYNOIO_GET_BACKPLANE_STATUS:
		DBGMESG("synobios_ioctl: SYNOIO_GET_BACKPLANE_STATUS\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_SET_UART2:
		DBGMESG("synobios_ioctl: SYNOIO_SET_UART2\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_GET_CPU_TEMPERATURE:
		DBGMESG("synobios_ioctl: SYNOIO_GET_CPU_TEMPERATURE\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_SET_CPU_FAN_STATUS:
		DBGMESG("synobios_ioctl: SYNOIO_SET_CPU_FAN_STATUS\n");
		{
			ret = -1;
			break;
		}
	case SYNOIO_CHECK_MICROP_ID:
		DBGMESG("synobios_ioctl: SYNOIO_CHECK_MICROP_ID\n");
		{
			ret = MICROP_ID_3611xs;
			break;
		}
		break;
	case SYNOIO_GET_EUNIT_TYPE: // Called intermittenly
		//DBGMESG("synobios_ioctl: SYNOIO_GET_EUNIT_TYPE\n");
	{
		EUNIT_PWRON_TYPE *pPwrOnType = (EUNIT_PWRON_TYPE *)arg;
		*pPwrOnType = EUNIT_NOT_SUPPORT;
		ret = 0;
	}
	break;
	case SYNOIO_SUPERIO_READ:
		DBGMESG("synobios_ioctl: SYNOIO_SUPERIO_READ\n");
		{
			ret = 0;
		}
		break;
	case SYNOIO_SUPERIO_WRITE:
		DBGMESG("synobios_ioctl: SYNOIO_SUPERIO_WRITE\n");
		ret = 0;
		break;
	case SYNOIO_IS_FULLY_SUPPORT_EUP:
		DBGMESG("synobios_ioctl: SYNOIO_IS_FULLY_SUPPORT_EUP\n");
		{
			SYNO_EUP_SUPPORT *pEupSupport = (SYNO_EUP_SUPPORT *)arg;
			// Warning: arg is null!
			//*pEupSupport = EUP_NOT_SUPPORT;
			ret = EUP_NOT_SUPPORT;
		}
		break;
	default:
		ret = -ENOSYS;
		printk(KERN_INFO "synobios_ioctl: un-defined ioctl number %x\n", cmd);
		break;
	}
END:
	return ret;
}

int synobios_open(struct inode *inode, struct file *filp)
{
	return 0;
}

int synobios_release(struct inode *inode, struct file *filp)
{
	return 0;
}


struct file_operations synobios_fops = {
	poll:			synobios_poll,
	unlocked_ioctl:	synobios_ioctl,
	compat_ioctl	synobios_ioctl,
	open:			synobios_open,
	release:		synobios_release,
};

int synobios_init(void)
{
	int result;

	printk(KERN_INFO "synobios: load, major number %d\n", SYNOBIOS_MAJOR);
	result = register_chrdev(SYNOBIOS_MAJOR, "synobios", &synobios_fops);
	if (result < 0) {
		printk(KERN_INFO "synobios: can't set major number\n");
		return result;
	}
	return 0;
}

void synobios_cleanup(void)
{
	printk("synobios: unload\n");
	unregister_chrdev(SYNOBIOS_MAJOR, "synobios");
}

MODULE_AUTHOR("Jukolaut");
MODULE_DESCRIPTION("dummy synobios\n") ;
MODULE_LICENSE("GPL");

module_init(synobios_init);
module_exit(synobios_cleanup);

